@@ -3,8 +3,8 @@
 ;; Copyright © 2025 Sergio Pastor Pérez
 ;;
 ;; Author: Sergio Pastor Pérez <sergio.pastorperez@gmail.com>
-;; Package-Version: 20260123.1457
-;; Package-Revision: 360bc05f630a
+;; Package-Version: 20260129.2241
+;; Package-Revision: 858b630b49a9
 ;; Package-Requires: ((emacs "30.1") (magit-section "4.3.8"))
 ;; Keywords: blue, tools
 ;; URL: https://codeberg.org/lapislazuli/blue.el
@@ -161,15 +161,6 @@ This is used when passing universal prefix argument `C-u' to
   (setq-local outline-regexp "▶")
   (read-only-mode 1))
 
-(defun blue--get-log-buffer ()
-  "Return `blue--log-buffer'."
-  (let ((buf (get-buffer blue--log-buffer)) ; Already existing.
-        (buf* (get-buffer-create blue--log-buffer)))
-    (unless buf
-      (with-current-buffer buf*
-        (blue-log-mode)))
-    buf*))
-
 (defun blue--get-build-dir ()
   "Return `blue--build-dir' if it exists, nil otherwise."
   (if (and blue--build-dir
@@ -199,14 +190,16 @@ This is used when passing universal prefix argument `C-u' to
 (defun blue--find-blueprint (&optional path)
   "Return path to top-level `blueprint.scm'.
 If PATH is non-nil, locate `blueprint.scm' from PATH."
-  (if-let* ((blueprint (locate-dominating-file (or path default-directory)
-                                               "blueprint.scm")))
+  (if-let* ((blueprint (or (getenv "BLUE_BLUEPRINT")
+                           (concat
+                            (or (locate-dominating-file (or path default-directory)
+                                                        "blueprint.scm")
+                                (error "Could not find blueprint.scm in current directory or parents"))
+                            "/blueprint.scm"))))
       ;; This expands resolving symlinks. It's needed since in the build
       ;; directory ther can be a symlinked 'blueprint.scm' for projects that
       ;; require configuration.
-      (file-truename
-       (directory-file-name
-        (concat blueprint "/blueprint.scm")))
+      (file-truename (directory-file-name blueprint))
     (error (concat "[BLUE] Failed to locate `blueprint.scm'"
                    (when path
                      (concat " in " path))))))
@@ -223,34 +216,18 @@ If PATH is non-nil, locate `blueprint.scm' from PATH."
   "Return t if FILE exists and is readable."
   (and file (file-exists-p file) (file-readable-p file)))
 
-
-;;; Memoization.
-
-(defun blue--memoize (function)
-  "Return a memoized version of FUNCTION."
-  (let ((cache (make-hash-table :test 'equal)))
-    (lambda (&rest args)
-      (let ((result (gethash args cache 'cache-miss)))
-        (if (eq result 'cache-miss)
-            (let ((result (apply function args)))
-              (puthash args result cache)
-              result)
-          result)))))
-
-(defmacro blue--define-memoized (name arglist docstring &rest body)
-  "Define a memoized function NAME.
-See `defun' for the meaning of NAME ARGLIST DOCSTRING and BODY."
-  (declare (doc-string 3) (indent 2))
-  `(defalias ',name
-     (blue--memoize (lambda ,arglist ,@body))
-     ,(format "(%S %s)\n\n%s"
-              name
-              (mapconcat #'symbol-name arglist " ")
-              docstring)))
-
 
 ;;; Logging.
 
+(defun blue--get-log-buffer ()
+  "Return `blue--log-buffer'."
+  (let ((buf (get-buffer blue--log-buffer)) ; Already existing.
+        (buf* (get-buffer-create blue--log-buffer)))
+    (unless buf
+      (with-current-buffer buf*
+        (blue-log-mode)))
+    buf*))
+
 (defun blue--format-header (command)
   "Format COMMAND header for output buffer.
 
@@ -273,13 +250,17 @@ EXIT-CODE displays the status of the command."
 COMMAND is the command string that generated OUTPUT.
 EXIT-CODE is the return value of CMD."
   (with-current-buffer (blue--get-log-buffer)
-    (let ((inhibit-read-only t))
+    (let ((inhibit-read-only t)
+          (start (point-max)))
+      (goto-char start)
+      (when command (insert (blue--format-header command)))
+      (insert output)
+      (blue--apply-filter-in-region
+       start (point)
+       #'blue-prettify-compilation-filter)
+      (unless (equal (point) (line-beginning-position))
+        (insert "\n"))
       (save-excursion
-        (goto-char (point-min))
-        (when command (insert (blue--format-header command)))
-        (insert output)
-        (unless (equal (point) (line-beginning-position))
-          (insert "\n"))
         (when exit-code (insert (blue--format-footer exit-code)))))))
 
 
@@ -357,7 +338,7 @@ the build directory `blue--build-dir' exists before adding it."
 
 ;;; Deserialization.
 
-(defun blue--execute-deserialize (options commands)
+(defun blue--execute (options commands)
   "Execute BLUE serialization COMMANDS with OPTIONS and return parsed output.
 
 If RAW is non nil, the serialized string will not be evaluated."
@@ -366,12 +347,11 @@ If RAW is non nil, the serialized string will not be evaluated."
          (args (append (or options '()) (cons "--color=always" commands)))
          (command-string (string-join (cons blue-binary args) " "))
          exit-code
-         (raw-output (with-output-to-string
+         (output (with-output-to-string
                        (setq exit-code
                              (apply #'call-process blue-binary nil standard-output nil args))))
-         (propertized-output (ansi-color-apply raw-output))
-         (filtered-output (replace-regexp-in-string ";;;.*\n?" "" raw-output)))
-    (blue--log-output propertized-output command-string exit-code)
+         (filtered-output (replace-regexp-in-string ";;;.*\n?" "" output)))
+    (blue--log-output output command-string exit-code)
     (cons filtered-output exit-code)))
 
 (defun blue--get-data (blueprint)
@@ -381,7 +361,7 @@ If RAW is non nil, the serialized string will not be evaluated."
               (cmd '(".serialize-commands" "--"
                      ".serialize-execution-environment" "--"
                      ".serialize-ui"))
-              (output (blue--execute-deserialize options cmd))
+              (output (blue--execute options cmd))
               (data (car output))
               (exit-code (cdr output)))
     (if (zerop exit-code)
@@ -392,9 +372,7 @@ If RAW is non nil, the serialized string will not be evaluated."
               (re-search-forward "\\(\\[.*\\]\\)\\({.*}\\)\\(\\[.*\\]\\)")
             (search-failed
              (blue--log-output (error-message-string err) "[BLUE] `re-search-forward'" 1)
-             (with-current-buffer blue--log-buffer
-               (goto-char (point-min)))
-             (display-buffer blue--log-buffer)
+             (display-buffer (blue--get-log-buffer))
              (error "[BLUE] Deserialization failed")))
           (let ((commands (match-string-no-properties 1))
                 (exec-env (match-string-no-properties 2))
@@ -406,9 +384,7 @@ If RAW is non nil, the serialized string will not be evaluated."
                                          :null-object nil
                                          :false-object nil))
                     (list commands exec-env ui))))
-      (with-current-buffer blue--log-buffer
-        (goto-char (point-min)))
-      (display-buffer blue--log-buffer)
+      (display-buffer (blue--get-log-buffer))
       (error "[BLUE] Deserialization failed"))))
 
 (defun blue--config-get (var config)
@@ -543,6 +519,128 @@ COMINT-P selects `comint-mode' for compilation buffer."
                     (blue--set-search-path))))))
     (compilation-start command comint-p)))
 
+(defun blue--visit-location (file &optional line column)
+  "Open FILE and move point to LINE and COLUMN if provided."
+  (when-let* ((path (if (file-exists-p file)
+                        file
+                      (locate-file file blue--search-path))))
+    (when (file-exists-p path)
+      (find-file path)
+      (when line
+        (goto-char (point-min))
+        (forward-line (1- line)))
+      (when column
+        (move-to-column column)))))
+
+(defun blue-open-hyperlink (url)
+  "Open a hyperlink URL, handling file:// URLs specially."
+  (cond
+   ((string-match
+     (rx bos "file://"
+         (* (not "/"))                           ; hostname (optional)
+         (group "/" (*? anything))               ; path (capture group 1)
+         (? "#L"                                 ; optional line number
+            (group (+ digit))                    ; line (capture group 2)
+            (? ":" (group (+ digit))))           ; optional column (capture group 3)
+         eos)
+     url)
+    (let* ((path (match-string 1 url))
+           (line-str (match-string 2 url))
+           (line (when line-str (string-to-number line-str)))
+           (col-str (match-string 3 url))
+           (col (when col-str (string-to-number col-str))))
+      (blue--visit-location path line col)))
+   (t (browse-url url))))
+
+(defun blue-ansi-buttonize-hyperlinks (beg end)
+  "Convert ANSI OSC 8 hyperlink sequences into clickable buttons in the region."
+  (save-excursion
+    (goto-char beg)
+    (while (re-search-forward
+            (rx "\033]8;;"
+                (group (* (not (any "\033" "\007"))))  ; URL capture group
+                (or "\033\\" "\007"))                  ; Terminator
+            end t)
+      (let ((url (match-string-no-properties 1))
+            (open-start (match-beginning 0))
+            (open-end (match-end 0)))
+        ;; Delete the opening sequence
+        (delete-region open-start open-end)
+        ;; Adjust end boundary
+        (setq end (- end (- open-end open-start)))
+        ;; Now point is at link-start (where the text begins)
+        (let ((link-start (point)))
+          ;; Find the closing sequence
+          (when (re-search-forward
+                 (rx "\033]8;;" (or "\033\\" "\007"))
+                 end t)
+            (let ((close-start (match-beginning 0))
+                  (close-end (match-end 0)))
+              ;; Delete the closing sequence
+              (delete-region close-start close-end)
+              ;; Adjust end boundary again
+              (setq end (- end (- close-end close-start)))
+              ;; Make the text between them a button
+              (when (and url (> close-start link-start))
+                (make-text-button link-start close-start
+                                  'action `(lambda (_)
+                                             (blue-open-hyperlink ,url))
+                                  'follow-link t
+                                  'help-echo (format "Click to open %s" url))))))))))
+
+(defun blue-hyperlinks-compilation-filter ()
+  "Translate OSC hyperlink escape sequences button text properties."
+  (let ((inhibit-read-only t))
+    (blue-ansi-buttonize-hyperlinks compilation-filter-start (point))))
+
+(defun blue-prettify-compilation-filter ()
+  "Combination of filters to prettify output in compilation buffers."
+  ;; HACK: apply first our own OSC filter since the built-in one,
+  ;; `ansi-osc-compilation-filter', does not visit lines properly. This
+  ;; ensures our text properties are applied before the OSC filter does
+  ;; anything.
+  (blue-hyperlinks-compilation-filter)
+  (ansi-color-compilation-filter)
+  (ansi-osc-compilation-filter))
+
+(defun blue--apply-filter-in-region (beg end filter)
+  "Helper to apply FILTER to string STR."
+  (save-excursion
+    (let ((compilation-filter-start beg))
+      (goto-char end) ; Filters take point as ending possition.
+      (funcall filter))))
+
+(defun blue--run-comint-filter (filter)
+  "Helper function to run FILTER in a comint buffer."
+  (let ((start-marker (if (and (markerp comint-last-output-start)
+			                   (eq (marker-buffer comint-last-output-start)
+				                   (current-buffer))
+			                   (marker-position comint-last-output-start))
+			              comint-last-output-start
+			            (point-min-marker)))
+	    (end-marker (process-mark (get-buffer-process (current-buffer)))))
+    (funcall filter start-marker end-marker)))
+
+(defun blue-prettify-comint-filter (_)
+  "Combination of filters to prettify output in comint buffers."
+  (dolist (filter '(blue-ansi-buttonize-hyperlinks
+                    ansi-color-apply-on-region
+                    ansi-osc-apply-on-region))
+    (blue--run-comint-filter filter)))
+
+;;;###autoload
+(define-minor-mode blue-prettify-compilation-mode
+  "Minor mode to enhance compilation mode's prettifying functions."
+  :global t
+  :group 'blue
+  :lighter " blue-pc"
+  (if blue-prettify-compilation-mode
+      (progn
+        (add-hook 'compilation-filter-hook #'blue-prettify-compilation-filter)
+        (add-hook 'comint-output-filter-functions #'blue-prettify-comint-filter))
+    (remove-hook 'compilation-filter-hook #'blue-prettify-compilation-filter)
+    (remove-hook 'comint-output-filter-functions #'blue-prettify-comint-filter)))
+
 
 ;;; Command Analysis.
 
